/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <time.h>
//#include <values.h>
#include "TFDialogGenSSynth.h"
#include "HeightFieldGenSSynth.h"
#include "MathRandom.h"
#include "TFGui.h"
#include "TFWindowHandler.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "agename.h"




/*
 *  constructor: initialize all data members 
 */
TFDialogGenSSynth::TFDialogGenSSynth ()
	        : GuiDialogOAC ("Specral Synthesis Generate Dialog", 
				"Generation Parameters"),
		  d_vbList (TRUE, 5),
		  d_hbSize (TRUE, 5),
		  d_hbDim (TRUE, 5),
		  d_hbInvert (TRUE, 5),
		  d_hbGenSeed (TRUE, 5),
		  d_hbSeed (TRUE, 5)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogGenSSynth\n");

	d_seedInitStr = new char [40];
	reset ();

	d_lblSize = new Gtk_Label ("Size");
	d_lblDim = new Gtk_Label ("Fractal dimension");
	d_lblInvert = new Gtk_Label ("Invert sin/cos");
	d_lblGenSeed = new Gtk_Label ("Generate new seed");
	d_lblSeed = new Gtk_Label ("SpecifySeed");
	d_adjDim = new Gtk_Adjustment (2.5, 2.0, 2.99, 1);
	d_adjSize = new Gtk_Adjustment (400, 10, 1000, 1);
	d_hsDim = new Gtk_HScale (*d_adjDim);
	d_hsSize = new Gtk_HScale (*d_adjSize);

	buildDialogWindow ();
	connect_to_method (d_btnGenSeed.clicked, this, &TFDialogGenSSynth::checkboxCallback);
}



/*
 *  destructor: clean up, free allocated memory
 */
TFDialogGenSSynth::~TFDialogGenSSynth ()
{
	if (d_seedInitStr) delete d_seedInitStr;

	delete d_lblSize;
	delete d_lblDim;
	delete d_lblInvert;
	delete d_lblGenSeed;
	delete d_lblSeed;
	delete d_adjDim;
	delete d_adjSize;
	delete d_hsDim;
	delete d_hsSize;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogGenSSynth\n");
}


/*
 *  reset: clear d_tfWin so the next generate will register a new 
 * 	window. Should be called before re-showing an existing dialog
 */
void TFDialogGenSSynth::reset ()
{
	d_HF = NULL;
	d_tfWin = NULL;
	this->set_title ("Generate Dialog:Spectral Synthesis");
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogGenSSynth::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogGenSSynth called without Frame label",
		"TFDialogGenSSynth::fillVBox");

        d_hbSize.pack_start (*d_lblSize, TRUE, TRUE, s_HBOff);
        d_lblSize->show ();
        d_hsSize->set_update_policy (GTK_UPDATE_DELAYED);
	d_hsSize->set_adjustment (*d_adjSize);
        d_hsSize->set_digits (0);
        d_hsSize->set_draw_value (TRUE);
        d_hbSize.pack_end (*d_hsSize, TRUE, TRUE, s_HBOff);
        d_hsSize->show ();
	d_vbList.pack_start (d_hbSize, TRUE, TRUE, s_VBOff);
	d_hbSize.show ();

        d_hbDim.pack_start (*d_lblDim, TRUE, TRUE, s_HBOff);
        d_lblDim->show ();
        d_hsDim->set_update_policy (GTK_UPDATE_DELAYED);
	d_hsDim->set_adjustment (*d_adjDim);
        d_hsDim->set_digits (2);
        d_hsDim->set_draw_value (TRUE);
        d_hbDim.pack_end (*d_hsDim, TRUE, TRUE, s_HBOff);
        d_hsDim->show ();
	d_vbList.pack_start (d_hbDim, TRUE, TRUE, s_VBOff);
	d_hbDim.show ();

	d_hbInvert.pack_start (*d_lblInvert, TRUE, TRUE, s_HBOff);
	d_lblInvert->show ();
	d_hbInvert.pack_end (d_btnInvert, TRUE, TRUE, s_HBOff);
	d_btnInvert.show ();
	d_vbList.pack_start (d_hbInvert, TRUE, TRUE, s_VBOff);
	d_hbInvert.show ();

	d_vbList.pack_start (d_sepMain, TRUE, TRUE, s_HBOff);
	d_sepMain.show ();

	d_hbGenSeed.pack_start (*d_lblGenSeed, TRUE, TRUE, s_HBOff);
	d_lblGenSeed->show ();
	d_hbGenSeed.pack_end (d_btnGenSeed, TRUE, TRUE, s_HBOff);
	d_btnGenSeed.set_active (TRUE);
	d_btnGenSeed.show ();
	d_vbList.pack_start (d_hbGenSeed, TRUE, TRUE, s_VBOff);
	d_hbGenSeed.show ();

	d_lblSeed->set_justify (GTK_JUSTIFY_RIGHT);
	d_hbSeed.pack_start (*d_lblSeed, TRUE, TRUE, s_HBOff);
	d_lblSeed->set_sensitive (FALSE);
	d_lblSeed->show ();

	MathRandom	*mRand = new MathRandom ();
	sprintf (d_seedInitStr, "%d", (int)mRand->rnd());
	delete mRand;
	d_enSeed.set_text (d_seedInitStr);
	d_enSeed.set_sensitive (FALSE);
	d_hbSeed.pack_end (d_enSeed, TRUE, TRUE, s_HBOff);
	d_enSeed.show ();
	d_vbList.pack_start (d_hbSeed, TRUE, TRUE, s_VBOff);
	d_hbSeed.show ();

	this->d_baseFrame->add (d_vbList);
	d_vbList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
	
}


/*
 *  buildDialogWindow:
 */
void TFDialogGenSSynth::buildDialogWindow ()
{
	this->set_usize (300, 280);
	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogGenSSynth::checkboxCallback ()
{
	bool		status=d_enSeed.is_sensitive ();

	d_lblSeed->set_sensitive (!status);
	d_enSeed.set_sensitive (!status);
}


void TFDialogGenSSynth::buttonCallbackApply ()
{
	float			H, dim; 	// Dimension = 3-h
	char			*s;
	int			size, seed;
	bool			invert, magnitude, HFisNew=FALSE;
	HeightFieldGenSSynth	*HFGen = NULL;


	if (!d_HF)
		{
		d_HF = new HeightField ();
		HFisNew = TRUE;
		}
	HFGen = new HeightFieldGenSSynth (d_HF);

	// get slider values
	dim = d_adjDim->get_value (); 	// 2 .. 3
	H = 3-dim;
	size = (int)(d_adjSize->get_value());

	// either generate or read a random number seed
	if (d_btnGenSeed.get_active())
		{
		MathRandom	*mRand = new MathRandom ();

		seed = (int) (mRand->rnd()*G_MAXINT);
		sprintf (d_seedInitStr, "%d", seed);
		d_enSeed.set_text (d_seedInitStr);
		delete mRand;
		}
	else
		{
		s = const_cast<char*>(d_enSeed.get_text().c_str());
		seed = atoi (s);
		}

	invert = d_btnInvert.get_active ();
	magnitude = d_btnGenSeed.get_active ();

	SanityCheck::bailout ((!HFGen), "HFGen==NULL", "TFDialogGenSSynth::buttonCallbackApply");
	HFGen->generate (size, H, seed, invert);
	delete HFGen;
	HFGen = NULL;

	if (HFisNew)
		{
		d_tfWin = TFGui::addHeightField (d_HF);
		this->set_title (d_HF->getName());
		}
	else
		d_tfWin->drawHeightField ();

	this->d_applyHit = TRUE;
}


gint TFDialogGenSSynth::delete_event_impl (GdkEventAny *e)
{
	this->hide ();
	return (0);
}


