/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogMerge.h"
#include "GuiDialogPreview.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "MathTrig.h"
#include "TFGui.h"
#include "TFWindowHandler.h"			// circular 


/*
 *  constructor: initialize all data members 
 */
TFDialogMerge::TFDialogMerge (FlexArray *winList)
	        : GuiDialogOAC ("Terraform Merge Dialog", "Merge Parameters"),
		  d_vBox1 (TRUE, 0),
		  d_vBox2 (FALSE, 0),
		  d_vBox3 (TRUE, 0),
		  d_vBox4 (TRUE, 0),
		  d_hbMain (FALSE, 10),
		  d_hbFactor1 (TRUE, 5),
		  d_hbFactor2 (TRUE, 5)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogMerge\n");

	SanityCheck::bailout ((!winList), "winList==NULL", "TFDialogMerge::TFDialogMerge");

	d_lastSelect = -1;
	d_winList = winList;

	d_lblFactor1 = new Gtk_Label ("Center X"),
	d_lblFactor2 = new Gtk_Label ("Center Y"),

	d_adjFactor1 = new Gtk_Adjustment (1.0, 0.01, 1, 0.01),
	d_adjFactor2 = new Gtk_Adjustment (1.0, 0.01, 1, 0.01),

	d_hsFactor1 = new Gtk_HScale (*d_adjFactor1),
	d_hsFactor2 = new Gtk_HScale (*d_adjFactor2),

	d_cList = NULL;
	d_cmbMethod = NULL;
	d_glstMethod = NULL;
	d_scrolledWin = NULL;
	d_previewFrame1 = d_previewFrame2 = d_previewFrame3 = NULL;
	d_drawArea1 = d_drawArea2 = d_drawArea3 = NULL;
	d_pixmap1 = d_pixmap2 = NULL;
	d_HF1 = d_HF2 = d_HF3 = NULL;
	d_HFD1 = d_HFD2 = d_HFD3 = NULL;
	d_drawArea1 = d_drawArea2 = d_drawArea3 = NULL;

	buildDialogWindow ();
	initHeightFields ();
	//sprintf (this->d_winTitle, "Merge: %s", d_HF1->getName());
	//this->set_title (this->d_winTitle);
	GuiDialogPreview::iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjFactor1->value_changed, this, &TFDialogMerge::updatePreviewCallback);
	connect_to_method (d_adjFactor2->value_changed, this, &TFDialogMerge::updatePreviewCallback);
	connect_to_method (d_cmbMethod->get_list()->selection_changed, this, &TFDialogMerge::updatePreviewCallback);
	connect_to_method (d_cList->select_row, this, &TFDialogMerge::rowSelectedCallback);
	connect_to_method (d_btn1.clicked, this, &TFDialogMerge::button1Callback);
	connect_to_method (d_btn2.clicked, this, &TFDialogMerge::button2Callback);
	//connect_to_method (d_cList->button_press_event, this, &TFDialogMerge::processCListButtonPress);
}


/*
 *  destructor: clean up 
 */
TFDialogMerge::~TFDialogMerge ()
{
	delete d_lblFactor1;
	delete d_lblFactor2;
	delete d_adjFactor1;
	delete d_adjFactor2;
	delete d_hsFactor1;
	delete d_hsFactor2;
	delete d_scrolledWin;
	delete d_previewFrame1;
	delete d_previewFrame2;
	delete d_previewFrame3;
	delete d_cmbMethod;
	delete d_cList;
	delete d_drawArea1;
	delete d_drawArea2;
	delete d_drawArea3;
	delete d_pixmap1;
	delete d_pixmap2;
	delete d_packSourceMain;
	delete d_packSourceGrp1;
	delete d_packSourceGrp2;
	delete d_packAssignButtons;

	if (d_HF1Preview)
		delete d_HF1Preview;
	if (d_HF2Preview)
		delete d_HF2Preview;
	if (d_HF3Preview)
		delete d_HF3Preview;

	if (d_HFD1Preview)
		delete d_HFD1Preview;
	if (d_HFD2Preview)
		delete d_HFD2Preview;
	if (d_HFD3Preview)
		delete d_HFD3Preview;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogMerge\n");
}


/*
 *  initHeightFields: initialize the HeightFields
 */
void TFDialogMerge::initHeightFields (TFWindowHandler *TFWHseed)
{
	SanityCheck::bailout ((!d_cList), "d_cList==NULL", "TFDialogMerge::initHeightFields");

	int 			c=0;
	TFWindowHandler		*TFWH;
	HeightFieldOps		*HFO;

	if (d_cList->rows() == 0)
		return;

	// get HF and HFD from CList row data items
	if (TFWHseed)
		TFWH = TFWHseed;
	else
		TFWH = static_cast<TFWindowHandler*>(d_cList->get_row_data(0));
	d_HF1 = TFWH->getHeightField();
	d_HFD1 = TFWH->getHeightFieldDraw();

	if (d_cList->rows() == 1)
		{
		d_HF2 = d_HF1;
		d_HFD2 = d_HFD1;
		}
	else
		{
		if (TFWHseed)
		    {
		    // find first HF in CList which is != TFWHseed
		    do 
			{
			TFWH = static_cast<TFWindowHandler*>(d_cList->get_row_data(c++));
			}
		    while (TFWH == TFWHseed);
		    }
		else
			TFWH = static_cast<TFWindowHandler*>(d_cList->get_row_data(1));

		d_HF2 = TFWH->getHeightField();
		d_HFD2 = TFWH->getHeightFieldDraw();
		}

	// clean up 
	if (d_HF1Preview)
		delete d_HF1Preview;
	if (d_HF2Preview)
		delete d_HF2Preview;
	if (d_HF3Preview)
		delete d_HF3Preview;

	if (d_HFD1Preview)
		delete d_HFD1Preview;
	if (d_HFD2Preview)
		delete d_HFD2Preview;
	if (d_HFD3Preview)
		delete d_HFD3Preview;

	// create the reduced Preview HFs and create HFDs for them
	d_frameGrp1->set_label (d_HF1->getName());
	HFO = new HeightFieldOps (d_HF1);
	d_HF1Preview = HFO->hfResize (100, 100);
	delete HFO;
	d_HFD1Preview = new HeightFieldDraw (d_HF1Preview, d_drawArea1);
	d_HFD1Preview->setMode ("0");
	d_HFD1Preview->setColormap (d_HFD1->getColormap());

	d_frameGrp2->set_label (d_HF2->getName());
	HFO = new HeightFieldOps (d_HF2);
	d_HF2Preview = HFO->hfResize (100, 100);
	delete HFO;
	d_HFD2Preview = new HeightFieldDraw (d_HF2Preview, d_drawArea2);
	d_HFD2Preview->setMode ("0");
	d_HFD2Preview->setColormap (d_HFD2->getColormap());

	d_HF3Preview = HeightFieldOps::merge('+',d_HF1Preview,d_HF2Preview,1,1);
	d_HFD3Preview = new HeightFieldDraw (d_HF3Preview, d_drawArea3);
	d_HFD3Preview->setMode ("0");
	d_HFD3Preview->setColormap (d_HFD1->getColormap());

	Gtk_Main *gtkMain = Gtk_Main::instance();
        while (gtkMain->events_pending())
                gtkMain->iteration ();

	d_HFD1Preview->draw ();
	d_HFD2Preview->draw ();
	d_HFD3Preview->draw ();
}


/*
 *  rebuildCList: rebuild the CList message from the TFGui:: source
 */
void TFDialogMerge::rebuildCList ()
{
	FlexArray	*winList = TFGui::getWindowList ();
	TFWindowHandler	*TFWH;
	char		*name;
	int		i, row, lim;

	SanityCheck::bailout ((!d_cList), "d_cList==NULL", "TFDialogMerge::rebuildCList");
	SanityCheck::bailout ((!winList), "winList==NULL", "TFDialogMerge::rebuildCList");

	lim = winList->getSize ();
	d_cList->freeze ();
	d_cList->clear();
	for (i=0; i<lim; i++)
		{
		TFWH = static_cast<TFWindowHandler*>(winList->El(i));
		name = TFWH->getHeightField()->getName ();
		row = d_cList->append (const_cast<const char**>(&name));
		d_cList->set_row_data (row, TFWH);
		}
	d_cList->thaw ();
}


/*
 *  insertCList: add the clist to the first vbox
 */
void TFDialogMerge::insertCList ()
{
	d_scrolledWin = new Gtk_ScrolledWindow ();
	d_scrolledWin->set_border_width (2);
	d_scrolledWin->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	d_cList = new Gtk_CList (1);
	d_cList->set_column_title (0, "Terrain Object List");
	d_cList->column_titles_show ();
	d_cList->set_shadow_type (GTK_SHADOW_OUT);

	d_scrolledWin->add (*d_cList);
	d_vBox1.pack_start (*d_scrolledWin, TRUE, TRUE, 0);
	d_hbMain.pack_start (d_vBox1, TRUE, TRUE, 10);

	d_cList->show ();
	d_scrolledWin->show ();
	d_vBox1.show ();
	d_hbMain.show();
}


/*
 *  insertAssignButtons: insert the buttons you press to assigb a height field
 */
void TFDialogMerge::insertAssignButtons ()
{
	string		*s;
	char		t[256];

	d_packAssignButtons = new Gtk_Packer (),
	sprintf (t, "%s/%s", TF_DATADIR, "arrow_right.xpm");
	s = new string (t);
	d_pixmap1 = new Gtk_Pixmap (*this, *s);
	d_btn1.add (d_pixmap1);
	d_pixmap1->show ();
	d_packAssignButtons->add (d_btn1, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_PACK_EXPAND);
	d_btn1.show ();

	d_pixmap2 = new Gtk_Pixmap (*this, *s);
	d_btn2.add (d_pixmap2);
	d_pixmap2->show ();
	d_packAssignButtons->add (d_btn2, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_PACK_EXPAND);
	d_btn2.show ();
	delete s;

	d_hbMain.pack_start (*d_packAssignButtons, FALSE, FALSE, 10);
	d_packAssignButtons->show ();
	d_hbMain.show();
}


/*
 *  insertSourcePreviews: insert the preview into the main frame/vbox
 */
void TFDialogMerge::insertSourcePreviews ()
{

	d_packSourceMain = new Gtk_Packer (),
	d_packSourceGrp1 = new Gtk_Packer (),
	d_packSourceGrp2 = new Gtk_Packer ();
	d_frameGrp1 = new Gtk_Frame ("Source 1"),
	d_frameGrp2 = new Gtk_Frame ("Source 2");

	// bounding frame for 1st group
	d_frameGrp1->add (*d_packSourceGrp1);
	d_frameGrp1->set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frameGrp1->set_border_width (10);
	d_packSourceGrp1->show ();

	// build 1st source preview drawing area 
	d_previewFrame1 = new Gtk_Frame ();
	d_previewFrame1->set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_previewFrame1->set_border_width (2);
	d_packSourceGrp1->add (*d_previewFrame1, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, (GtkPackerOptions)0);
	d_previewFrame1->show ();

	d_drawArea1 = new GuiBufferedDrawingArea ();
	d_drawArea1->setSync (0);
	d_previewFrame1->add (*d_drawArea1);
	d_drawArea1->show ();
	d_drawArea1->set_usize (100, 100);

	// hscale for 1st preview 
	d_hsFactor1->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsFactor1->set_digits (2);
	d_hsFactor1->set_draw_value (TRUE);
	d_packSourceGrp1->add (*d_hsFactor1, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
	d_hsFactor1->show ();
	d_packSourceMain->add (*d_frameGrp1, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
	d_frameGrp1->show ();


	// build dropdown list of operations
	d_cmbMethod = new Gtk_Combo ();
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_ADD));
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_SUB));
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_MULT));
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_DIV));
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_MIN));
	d_glstMethod = g_list_append (d_glstMethod, const_cast<char*>(MENU_MERGE_MAX));
	d_cmbMethod->set_popdown_strings (d_glstMethod);
	d_cmbMethod->get_entry()->set_text (MENU_MERGE_ADD);
	d_cmbMethod->get_entry()->set_editable (FALSE);
	//d_vBox2.pack_start (*d_cmbMethod, FALSE, FALSE, 0);
	d_packSourceMain->add (*d_cmbMethod, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_PACK_EXPAND);
	d_cmbMethod->show ();


	// bounding frame for 2nd group
	d_frameGrp2->add (*d_packSourceGrp2);
	d_frameGrp2->set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frameGrp2->set_border_width (10);
	d_packSourceGrp2->show ();

	// build 2nd source preview drawing area 
	d_previewFrame2 = new Gtk_Frame ();
	d_previewFrame2->set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_previewFrame2->set_border_width (10);
	d_packSourceGrp2->add (*d_previewFrame2, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, (GtkPackerOptions)0);
	d_previewFrame2->show ();

	d_drawArea2 = new GuiBufferedDrawingArea ();
	d_drawArea2->setSync (0);
	d_previewFrame2->add (*d_drawArea2);
	d_drawArea2->show ();
	d_drawArea2->set_usize (100, 100);

	// hscale for 2nd preview 
	d_hsFactor2->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsFactor2->set_digits (2);
	d_hsFactor2->set_draw_value (TRUE);
	d_packSourceGrp2->add (*d_hsFactor2, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
	d_hsFactor2->show ();
	d_packSourceMain->add (*d_frameGrp2, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
	d_frameGrp2->show ();


	d_hbMain.pack_start (*d_packSourceMain, FALSE, FALSE, 10);
	d_packSourceMain->show ();
}


/*
 *  insertDestPreviews: insert the preview into the main frame/vbox
 */
void TFDialogMerge::insertDestPreviews ()
{
	Gtk_Frame	*destFrame = new Gtk_Frame ("Result");
	Gtk_VBox	*destVBox = new Gtk_VBox (TRUE, 5);

	// build title frame
	destVBox->pack_start (*destFrame, FALSE, FALSE, 10);
	destFrame->show ();
	destFrame->set_border_width (5);
	destFrame->add (d_vBox3);
	d_vBox3.show ();

	// build dest preview drawing area 
	d_previewFrame3 = new Gtk_Frame ();
	d_previewFrame3->set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_previewFrame3->set_border_width (2);
	d_vBox3.pack_start (*d_previewFrame3, FALSE, FALSE, 0);
	d_previewFrame3->show ();

	d_drawArea3 = new GuiBufferedDrawingArea ();
	d_drawArea3->setSync (0);
	d_previewFrame3->add (*d_drawArea3);
	d_drawArea3->show ();
	d_drawArea3->set_usize (100, 100);

	d_hbMain.pack_start (*destVBox, FALSE, FALSE, 10);
	destVBox->show ();

}


/*
 *  buildDialogWindow: build the dialog window from it's components
 */
void TFDialogMerge::buildDialogWindow ()
{
	this->set_usize (600, 500);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);
	setupVBox ();

	SanityCheck::bailout ((!this->d_baseFrame), "this->d_baseFrame==NULL", "TFDialogMerge::buildDialogWindow");
	this->d_baseFrame->add (d_hbMain);

	insertCList ();
	insertAssignButtons ();
	insertSourcePreviews ();
	insertDestPreviews ();
	this->fillActionArea ();
	rebuildCList ();

	this->show ();
}


/*
 *  button1Callback: process the 1st callback button
 */
void TFDialogMerge::rowSelectedCallback (int row, int col, GdkEvent *e)
{
	if (TFGui::getWindowList()->getSize() != d_cList->rows())
		{
		rebuildCList ();
		initHeightFields ();
		}

	d_lastSelect = row;
}


/*
 *  button1Callback: process the 1st callback button
 */
void TFDialogMerge::button1Callback ()
{
	TFWindowHandler		*TFWH;
	HeightFieldOps		*HFO;

	if (TFGui::getWindowList()->getSize() != d_cList->rows())
		{
		rebuildCList ();
		initHeightFields ();
		}

	if (d_lastSelect == -1)
		return;

	TFWH = static_cast<TFWindowHandler*>(d_cList->get_row_data(d_lastSelect));
	d_HF1 = TFWH->getHeightField();
	d_frameGrp1->set_label (d_HF1->getName());
	HFO = new HeightFieldOps (d_HF1);
	d_HF1Preview = HFO->hfResize (100, 100);
	delete HFO;

	// allow frame title to change 
	Gtk_Main *gtkMain = Gtk_Main::instance();
        while (gtkMain->events_pending())
                gtkMain->iteration ();

	d_HFD1Preview = new HeightFieldDraw (d_HF1Preview, d_drawArea1);
	d_HFD1Preview->setMode ("0");
	d_HFD1Preview->setColormap (d_HFD1->getColormap());
	d_HFD1Preview->draw ();
	updatePreviewCallback ();
}


/*
 *  button2Callback: process the 2nd callback button
 */
void TFDialogMerge::button2Callback ()
{
	TFWindowHandler		*TFWH;
	HeightFieldOps		*HFO;

	if (TFGui::getWindowList()->getSize() != d_cList->rows())
		{
		rebuildCList ();
		initHeightFields ();
		}

	if (d_lastSelect == -1)
		return;

	TFWH = static_cast<TFWindowHandler*>(d_cList->get_row_data(d_lastSelect));
	d_HF2 = TFWH->getHeightField();
	d_frameGrp2->set_label (d_HF2->getName());
	HFO = new HeightFieldOps (d_HF2);
	d_HF2Preview = HFO->hfResize (100, 100);
	delete HFO;

	// allow frame title to change 
	Gtk_Main *gtkMain = Gtk_Main::instance();
        while (gtkMain->events_pending())
                gtkMain->iteration ();

	d_HFD2Preview = new HeightFieldDraw (d_HF2Preview, d_drawArea2);
	d_HFD2Preview->setMode ("0");
	d_HFD2Preview->setColormap (d_HFD2->getColormap());
	d_HFD2Preview->draw ();
	updatePreviewCallback ();
}


/*
 *  processCListButtonPress: process a button press on the CList
 */
gint TFDialogMerge::processCListButtonPress (GdkEventButton *e)
{
	return 0;
}


/*
 *  buttonCallbackApply: roughen the Height Field
 */
void TFDialogMerge::buttonCallbackApply ()
{
	char		op;
	float		factor1, factor2;
	string		entry;

	SanityCheck::bailout ((!d_cList), "d_cList==NULL", "TFDialogMerge::rebuildCList");
	SanityCheck::bailout ((!d_HF1), "d_HF1==NULL", "TFDialogMerge::updatePreviewCallback");
	SanityCheck::bailout ((!d_HF2), "d_HF2==NULL", "TFDialogMerge::updatePreviewCallback");
	SanityCheck::bailout ((!d_HF3Preview), "d_HF3Preview==NULL", "TFDialogMerge::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFD3Preview), "d_HFD3Preview==NULL", "TFDialogMerge::updatePreviewCallback");

	factor1 = d_adjFactor1->get_value ();
	factor2 = d_adjFactor2->get_value ();
	entry = (d_cmbMethod->get_entry())->get_text();
	if (entry == MENU_MERGE_ADD)
		op = '+';
	else
	if (entry == MENU_MERGE_SUB)
		op = '-';
	else
	if (entry == MENU_MERGE_MULT)
		op = '*';
	else
	if (entry == MENU_MERGE_DIV)
		op = '/';
	else
	if (entry == MENU_MERGE_MIN)
		op = 'm';
	else
	if (entry == MENU_MERGE_MAX)
		op = 'M';

	if (!(d_HF3 = HeightFieldOps::merge (op, d_HF1, d_HF2, factor1, factor2)))
		{
		SanityCheck::warning ((TRUE), "Height Field Merge failed ...\n", "TFDialogMerge::buttonCallbackApply");
		return;
		}

	TFGui::addHeightField (d_HF3);

	// only do this if window stays open 
	if (d_applyHit)
		updatePreviewCallback ();

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a slider has been updated.
 */
void TFDialogMerge::updatePreviewCallback ()
{
	char		op;
	float		factor1, factor2;
	string		entry;
	FlexArray	*winList = TFGui::getWindowList ();

	SanityCheck::bailout ((!d_cList), "d_cList==NULL", "TFDialogMerge::rebuildCList");
	SanityCheck::bailout ((!winList), "winList==NULL", "TFDialogMerge::rebuildCList");
	SanityCheck::bailout ((!d_HF3Preview), "d_HF3Preview==NULL", "TFDialogMerge::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFD3Preview), "d_HFD3Preview==NULL", "TFDialogMerge::updatePreviewCallback");

	if (winList->getSize() != d_cList->rows())
		{
		rebuildCList ();
		initHeightFields ();
		}

	factor1 = d_adjFactor1->get_value ();
	factor2 = d_adjFactor2->get_value ();
	entry = (d_cmbMethod->get_entry())->get_text();
	if (entry == MENU_MERGE_ADD)
		op = '+';
	else
	if (entry == MENU_MERGE_SUB)
		op = '-';
	else
	if (entry == MENU_MERGE_MULT)
		op = '*';
	else
	if (entry == MENU_MERGE_DIV)
		op = '/';
	else
	if (entry == MENU_MERGE_MIN)
		op = 'm';
	else
	if (entry == MENU_MERGE_MAX)
		op = 'M';

	if (d_HF3Preview)
		delete d_HF3Preview;
	if (d_HFD3Preview)
		delete d_HFD3Preview;

	d_HF3Preview = HeightFieldOps::merge (op, d_HF1Preview, d_HF2Preview, 
						factor1, factor2);
	d_HFD3Preview = new HeightFieldDraw (d_HF3Preview, d_drawArea3);
	d_HFD3Preview->setMode ("0");
	d_HFD3Preview->setColormap (d_HFD1->getColormap());
	d_HFD3Preview->draw ();
}


/*
 *  (virtual) delete_event_impl: provide a function to hande window close
 *                               request
 */
gint TFDialogMerge::delete_event_impl (GdkEventAny *e)
{
	this->hide ();
	return (0);
}

